/**
	Goebish's RGB filter APE 1.1
**/

#include <windows.h>
#include "resource.h"
#include "avs_ape.h"


#define MOD_NAME "Virtual Effect / RGB filter"
#define UNIQUEIDSTRING "Virtual Effect: RGB filter"


class C_THISCLASS : public C_RBASE 
{
	protected:
	public:
		C_THISCLASS();
		virtual ~C_THISCLASS();

		virtual int render(char visdata[2][2][576], int isBeat, int *framebuffer, int *fbout, int w, int h);
		
		virtual HWND conf(HINSTANCE hInstance, HWND hwndParent);
		virtual char *get_desc();

		virtual void load_config(unsigned char *data, int len);
		virtual int  save_config(unsigned char *data);
		virtual int pos(int x,int y,int l);

		int enabled;	// toggles red filter on and off
		int enabled2;	// toggles green filter on and off
		int enabled3;	// toggles blue filter on and off
		int enabled4;	// toggles on beat switching on and off
		bool membeat;	// memoire de beat
	};

// global configuration dialog pointer 
static C_THISCLASS *g_ConfigThis; 
// global DLL instance pointer (not needed in this example, but could be useful)
static HINSTANCE g_hDllInstance; 



// this is where we deal with the configuration screen
static BOOL CALLBACK g_DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:

			if (g_ConfigThis->enabled) 
			{
				CheckDlgButton(hwndDlg,IDC_CHECK1,BST_CHECKED);
			
			}
			if (g_ConfigThis->enabled2 )
			{
				CheckDlgButton(hwndDlg,IDC_CHECK2,BST_CHECKED);
			}
			if (g_ConfigThis->enabled3 )
			{
				CheckDlgButton(hwndDlg,IDC_CHECK3,BST_CHECKED);
			}
			if (g_ConfigThis->enabled4 )
			{
				CheckDlgButton(hwndDlg,IDC_CHECK4,BST_CHECKED);
			}
			return 1;

		case WM_COMMAND:
			
			// see if red filter checkbox is checked
			if (LOWORD(wParam) == IDC_CHECK1)
			{
				g_ConfigThis->enabled= (IsDlgButtonChecked(hwndDlg,IDC_CHECK1)?1:0);
			}
			
			// see if green filter checkbox is checked
			if (LOWORD(wParam) == IDC_CHECK2) 
			{
				     
				g_ConfigThis->enabled2 = (IsDlgButtonChecked(hwndDlg,IDC_CHECK2)?1:0);
			}
			// see if blue checkbox is checked
			if (LOWORD(wParam) == IDC_CHECK3) 
			{
				     
				g_ConfigThis->enabled3 = (IsDlgButtonChecked(hwndDlg,IDC_CHECK3)?1:0);
			}
			// see if on beat switch checkbox is checked
			if (LOWORD(wParam) == IDC_CHECK4) 
			{
				     
				g_ConfigThis->enabled4 = (IsDlgButtonChecked(hwndDlg,IDC_CHECK4)?1:0);
				g_ConfigThis->membeat=false;
			}
		  return 0;
	}
	return 0;
}




// set up default configuration 
C_THISCLASS::C_THISCLASS() 
{
	//red actif par defaut
	enabled=1;
	//green actif par defaut
	enabled2=1;
	//blue actif par defaut
	enabled3=1;
	//on beat switching inactif par defaut
	enabled4=0;
	//membeat a 0
	membeat=false;
}

// virtual destructor
C_THISCLASS::~C_THISCLASS() 
{
}


// RENDER FUNCTION:
// render should return 0 if it only used framebuffer, or 1 if the new output data is in fbout. this is
// used when you want to do something that you'd otherwise need to make a copy of the framebuffer.
// w and h are the-*/ width and height of the screen, in pixels.
// isBeat is 1 if a beat has been detected.
// visdata is in the format of [spectrum:0,wave:1][channel][band].

int C_THISCLASS::render(char visdata[2][2][576], int isBeat, int *framebuffer, int *fbout, int w, int h)
{
	
	// si r&g&b selectionns alors on s'en va...
	if (enabled && enabled2 && enabled3) 
	{
		return 0;
	}

	// si pas beat switching alors membeat=1
	if(!enabled4)
	{
		membeat=true;
	}
	
	// si un beat et beat switching alors on inverse membeat
	if(isBeat && enabled4)
	{
		membeat=!membeat;
	}

	// si onbeat switching et membeat a 0 alors on s'en va...
	if(enabled4 && !membeat)
	{
		return 0;
	}

	// on fait seulement qqch si membeat=1
	if(membeat)
	{

		int tot=w*h;
		// si r&g&b deselectionns alors image noire
		if (!enabled && !enabled2 && !enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=0;
			}
			return 1;
		}

		// si red seulement
		if (enabled && !enabled2 && !enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 16711680;
			}
			return 1;
		}

		// si green seulement
		if (!enabled && enabled2 && !enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 65280;
			}
			return 1;
		}

		// si blue seulement
		if (!enabled && !enabled2 && enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 255;
			}
			return 1;
		}

		// si green et blue seulement
		if (!enabled && enabled2 && enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 65535;
			}
			return 1;
		}

		// si red & blue seulement
		if (enabled && !enabled2 && enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 16711935;
			}
			return 1;
		}

		// si red & green seulement
		if (enabled && enabled2 && !enabled3)
		{
			for(int t=0;t<tot;t++)
			{
				fbout[t]=framebuffer[t] & 16776960;
			}
			return 1;
		}
	}
	return 0;
}

int C_THISCLASS::pos(int x,int y,int l)
{
	return x+(y*l);
}
	

HWND C_THISCLASS::conf(HINSTANCE hInstance, HWND hwndParent) // return NULL if no config dialog possible
{
	g_ConfigThis = this;
	return CreateDialog(hInstance,MAKEINTRESOURCE(IDD_CONFIG),hwndParent,g_DlgProc);
}


char *C_THISCLASS::get_desc(void)
{ 
	return MOD_NAME; 
}


// load_/save_config are called when saving and loading presets (.avs files)

#define GET_INT() (data[pos]|(data[pos+1]<<8)|(data[pos+2]<<16)|(data[pos+3]<<24))
void C_THISCLASS::load_config(unsigned char *data, int len) // read configuration of max length "len" from data.
{
	int pos=0;

	// always ensure there is data to be loaded
	if (len-pos >= 4) 
	{
		// load red filter activation toggle
		enabled=GET_INT();
		pos+=4; 
	}

	if (len-pos >= 4) 
    { 
		// load green filter activation toggle
		enabled2=GET_INT(); 
		pos+=4; 
	}

	if (len-pos >= 4) 
    { 
		// load blue activation toggle
		enabled3=GET_INT(); 
		pos+=4; 
	}

	if (len-pos >=4)
	{
		// load on beat switch activation toggle
		enabled4=GET_INT();
		pos+=4;
	}
}


// write configuration to data, return length. config data should not exceed 64k.
#define PUT_INT(y) data[pos]=(y)&255; data[pos+1]=(y>>8)&255; data[pos+2]=(y>>16)&255; data[pos+3]=(y>>24)&255
int  C_THISCLASS::save_config(unsigned char *data) 
{
	int pos=0;
	
	PUT_INT(enabled); 
	pos+=4;
		
	PUT_INT(enabled2); 
	pos+=4;

	PUT_INT(enabled3);
	pos+=4;

	PUT_INT(enabled4);
	pos+=4;

	return pos;
}







// export stuff
C_RBASE *R_RetrFunc(char *desc) // creates a new effect object if desc is NULL, otherwise fills in desc with description
{
	if (desc) 
	{ 
		strcpy(desc,MOD_NAME); 
		return NULL; 
	}
	return (C_RBASE *) new C_THISCLASS();
}

extern "C"
{
	__declspec (dllexport) int _AVS_APE_RetrFunc(HINSTANCE hDllInstance, char **info, int *create) // return 0 on failure
	{
		g_hDllInstance=hDllInstance;
		*info=UNIQUEIDSTRING;
		*create=(int)(void*)R_RetrFunc;
		return 1;
	}
};

